"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var domain_1 = require('../domain');
var file_system_1 = require('../io/file_system');
var serenity_1 = require('../serenity');
var path = require('path');
var stack_trace_1 = require('stack-trace');
var md5_1 = require('ts-md5/dist/md5');
var _ = require('lodash');
function jsonReporter(pathToReports) {
    if (pathToReports === void 0) { pathToReports = serenity_1.Default_Path_To_Reports; }
    return new JsonReporter(new file_system_1.FileSystem(pathToReports));
}
exports.jsonReporter = jsonReporter;
var JsonReporter = (function () {
    function JsonReporter(fs) {
        this.fs = fs;
    }
    JsonReporter.prototype.assignTo = function (stage) {
        this.stage = stage;
        this.stage.manager.registerInterestIn(JsonReporter.Events_of_Interest, this);
    };
    JsonReporter.prototype.notifyOf = function (event) {
        switch (event.constructor.name) {
            case domain_1.SceneStarts.name: return this.sceneStarts(event);
            case domain_1.ActivityStarts.name: return this.activityStarts(event);
            case domain_1.ActivityFinished.name: return this.activityFinished(event);
            case domain_1.SceneFinished.name: return this.sceneFinished(event);
            case domain_1.PhotoAttempted.name: return this.photoAttempted(event);
            default: break; // ignore any other events
        }
    };
    JsonReporter.prototype.sceneStarts = function (_a) {
        var value = _a.value, timestamp = _a.timestamp;
        this.rehearsal = new Rehearsal(value, timestamp);
    };
    JsonReporter.prototype.activityStarts = function (_a) {
        var value = _a.value, timestamp = _a.timestamp;
        this.rehearsal.activityStarts(value, timestamp);
    };
    JsonReporter.prototype.activityFinished = function (_a) {
        var value = _a.value, timestamp = _a.timestamp;
        this.rehearsal.activityFinished(value, timestamp);
    };
    JsonReporter.prototype.photoAttempted = function (_a) {
        var value = _a.value, timestamp = _a.timestamp;
        this.rehearsal.photoTaken(value, timestamp);
    };
    JsonReporter.prototype.sceneFinished = function (_a) {
        var _this = this;
        var value = _a.value, timestamp = _a.timestamp;
        var filename = md5_1.Md5.hashStr(value.subject.id) + ".json";
        this.rehearsal.finished(value, timestamp);
        this.stage.manager.informOfWorkInProgress(this.rehearsal.report().then(function (report) { return _this.fs.store(filename, JSON.stringify(report)); }));
    };
    JsonReporter.Events_of_Interest = [domain_1.SceneStarts, domain_1.ActivityStarts, domain_1.ActivityFinished, domain_1.PhotoAttempted, domain_1.SceneFinished];
    return JsonReporter;
}());
exports.JsonReporter = JsonReporter;
var Rehearsal = (function () {
    function Rehearsal(scene, timestamp) {
        var sceneReport = new SceneReport(scene, timestamp);
        this.current = sceneReport;
        this.previous = sceneReport;
    }
    Rehearsal.prototype.activityStarts = function (step, timestamp) {
        var activityReport = new ActivityReport(step, timestamp);
        this.current.append(activityReport);
        this.current = activityReport;
    };
    Rehearsal.prototype.activityFinished = function (outcome, timestamp) {
        this.current.completedWith(outcome, timestamp);
        this.previous = this.current;
        this.current = this.current.parent;
    };
    Rehearsal.prototype.finished = function (outcome, timestamp) {
        this.current.completedWith(outcome, timestamp);
    };
    Rehearsal.prototype.photoTaken = function (receipt, timestamp) {
        [this.previous, this.current]
            .filter(function (report) { return report instanceof ActivityReport; })
            .map(function (report) { return report; })
            .filter(function (report) { return report.concerns(receipt.activity); })
            .map(function (report) { return report.attachPhoto(receipt.photo); });
    };
    Rehearsal.prototype.report = function () {
        return this.current.toJSON();
    };
    return Rehearsal;
}());
var SerenityReport = (function () {
    function SerenityReport(startTimestamp) {
        this.children = [];
        this.startedAt = startTimestamp;
    }
    SerenityReport.prototype.append = function (stepExecutionReport) {
        var report = stepExecutionReport;
        report.parent = this;
        this.children.push(report);
    };
    SerenityReport.prototype.completedWith = function (outcome, finishedAt) {
        this.result = outcome.result;
        this.error = outcome.error;
        this.duration = finishedAt - this.startedAt;
    };
    SerenityReport.prototype.errorIfPresent = function () {
        if (!this.error) {
            return undefined; // so that the field is not rendered (that's what Serenity JVM expects for now)
        }
        return {
            errorType: this.error.name,
            message: this.error.message,
            stackTrace: this.stackTraceOf(this.error),
        };
    };
    SerenityReport.prototype.mapAll = function (items, mapper) {
        if (mapper === void 0) { mapper = function (x) { return x; }; }
        var onlyDefined = function (item) { return item !== undefined; };
        return Promise.all(items).then(function (all) { return all.filter(onlyDefined).map(mapper); });
    };
    SerenityReport.prototype.ifNotEmpty = function (list) {
        return !!list.length ? list : undefined;
    };
    // todo: extract a parser
    SerenityReport.prototype.stackTraceOf = function (error) {
        var serenityCode = /node_modules[\\/]serenity/, onlyIfFound = function (index) { return index !== -1 ? index : undefined; }, firstSerenityStackFrame = function (stack) { return onlyIfFound(stack.findIndex(function (frame) { return !!serenityCode.exec(frame.getFileName()); })); }, parsed = stack_trace_1.parse(error);
        return parsed.slice(0, firstSerenityStackFrame(parsed)).map(function (frame) {
            return {
                declaringClass: frame.getTypeName() || frame.getFunctionName() || '',
                methodName: frame.getMethodName() || frame.getFunctionName() || '',
                fileName: frame.getFileName(),
                lineNumber: frame.getLineNumber(),
            };
        });
    };
    return SerenityReport;
}());
var SceneReport = (function (_super) {
    __extends(SceneReport, _super);
    function SceneReport(scene, startTimestamp) {
        _super.call(this, startTimestamp);
        this.scene = scene;
    }
    SceneReport.prototype.toJSON = function () {
        var _this = this;
        return this.mapAll(this.children.map(function (r) { return r.toJSON(); })).then(function (serialisedChildren) {
            return {
                name: _this.scene.name,
                title: _this.scene.name,
                description: '',
                tags: _this.tagsFor(_this.scene),
                // driver                               // todo: missing
                testSource: 'cucumber',
                startTime: _this.startedAt,
                manual: false,
                duration: _this.duration,
                result: domain_1.Result[_this.result],
                testSteps: serialisedChildren,
                issues: _this.issuesCoveredBy(_this.scene),
                userStory: {
                    id: _this.dashify(_this.scene.category),
                    storyName: _this.scene.category,
                    path: path.relative(process.cwd(), _this.scene.path),
                    type: 'feature',
                },
                testFailureCause: _this.errorIfPresent(),
            };
        });
    };
    SceneReport.prototype.dashify = function (name) {
        var dashified = name
            .replace(/([a-z])([A-Z])/g, '$1-$2')
            .replace(/[ \t\W]/g, '-')
            .replace(/^-+|-+$/g, '');
        return dashified.toLowerCase();
    };
    SceneReport.prototype.isAnIssue = function (tag) {
        return ['issue', 'issues'].indexOf(tag.type) !== -1;
    };
    SceneReport.prototype.issuesCoveredBy = function (scene) {
        var onlyIssueTags = this.isAnIssue, toIssueIds = function (tag) { return tag.values; };
        return _.chain(scene.tags).filter(onlyIssueTags).map(toIssueIds).flatten().uniq().value();
    };
    SceneReport.prototype.tagsFor = function (scene) {
        var isAnIssue = this.isAnIssue;
        function serialise(tag) {
            var noValue = function (t) { return { name: t.type, type: 'tag' }; }, withValue = function (t) { return { name: t.values.join(','), type: t.type }; };
            return tag.values.length === 0
                ? noValue(tag)
                : withValue(tag);
        }
        function breakDownIssues(tag) {
            return isAnIssue(tag)
                ? tag.values.map(function (issueId) { return new domain_1.Tag('issue', [issueId]); })
                : tag;
        }
        function featureTag(featureName) {
            return {
                name: featureName,
                type: 'feature',
            };
        }
        return _.chain(scene.tags)
            .map(breakDownIssues)
            .flatten()
            .map(serialise)
            .uniqBy('name')
            .concat(featureTag(scene.category))
            .value();
    };
    return SceneReport;
}(SerenityReport));
var ActivityReport = (function (_super) {
    __extends(ActivityReport, _super);
    function ActivityReport(activity, startTimestamp) {
        _super.call(this, startTimestamp);
        this.activity = activity;
        this.promisedPhotos = [];
    }
    ActivityReport.prototype.concerns = function (activity) {
        return this.activity.equals(activity);
    };
    ActivityReport.prototype.attachPhoto = function (promisedPhoto) {
        this.promisedPhotos.push(promisedPhoto);
    };
    ActivityReport.prototype.completedWith = function (outcome, finishedAt) {
        _super.prototype.completedWith.call(this, outcome, finishedAt);
    };
    ActivityReport.prototype.toJSON = function () {
        var _this = this;
        return this.mapAll(this.promisedPhotos, this.serialise).then(function (serialisedPhotos) {
            return _this.mapAll(_this.children.map(function (r) { return r.toJSON(); })).then(function (serialisedChildren) {
                return {
                    description: _this.activity.name,
                    startTime: _this.startedAt,
                    duration: _this.duration,
                    result: domain_1.Result[_this.result],
                    children: serialisedChildren,
                    exception: _this.errorIfPresent(),
                    screenshots: _this.ifNotEmpty(serialisedPhotos),
                };
            });
        });
    };
    ActivityReport.prototype.serialise = function (photo) {
        return { screenshot: path.basename(photo.path) };
    };
    return ActivityReport;
}(SerenityReport));

//# sourceMappingURL=json_reporter.js.map
